/*

    Copyright (C) 2003 Bjorn Martins Paz (bmp@narpax.com.br)
    Copyright (C) 2003 Davi Einstein Melges Arnaut (dema@narpax.com.br)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TAdvMsgBox.h"
#include "TString.h"

/*----------------------------------------------------------------------------------------------*/
/* Constructor */
TTextBox::TTextBox(const TRect& bounds,
                   const char *sText,
                   const int nCols,
                   const int nRows,
                   TScrollBar *hSB,
                   TScrollBar *vSB) : TScroller(bounds, hSB, vSB) {
   int i = 0;
   ccIndex index = 0;
   TString line;

   m_nCols = nCols;
   m_nRows = nRows;

   growMode = gfGrowHiX | gfGrowHiY;
//   options = options | ofFramed;

   this->setLimit(m_nCols+1, m_nRows);

   m_Text = new TStringCollection(0,1);

   line.clean();
   line += " ";
   while( sText[i] != '\0' ) {
      if( sText[i] == '\n' ) {
         m_Text->atInsert( index++, line.cutData() );
         line.clean();
         line += " ";
         i++;
      }
      else {
         if( sText[i] == '\t' )
            line += TAB_STRING;
         else
            line += sText[i];
         i++;
      }
   }

   m_Text->atInsert( index, line.cutData() );
}

/*----------------------------------------------------------------------------------------------*/   
/* Draw text */
void TTextBox::draw(void) {
   int count, y = 0;
   TDrawBuffer buf;
   ccIndex index;
   ushort color;
   char *line;
   char *s;

   color = getColor(0x0301);
   s = new char[ this->size.x+1 ];

   count = m_Text->getCount();
   if( delta.y < count ) {
      index = delta.y;

      for( ; y < size.y ; y++ ) {
         line = (char *)m_Text->at(index);
         buf.moveChar( 0, ' ', color, size.x );

         if( delta.x > (int)strlen(line) )
            s[1] = '\0';                  
         else {
            strncpy( s, line+delta.x, size.x );
            s[size.x] = '\0';                    
         }

         buf.moveStr( 0, s, color );
         writeLine( 0, y, size.x, 1, buf );

         index++;
         if( index >= count )
            goto print_blank;
      }
   }

/* If we reach here, it's because there are no more lines
   in the list, or the window is already full. If there
   were no more lines in the list, then we fill the rest
   of the text-window with blank lines. */
print_blank:
   delete s;
   buf.moveChar( 0, ' ', color, size.x );
   for( ; y < size.y ; y++ )
      writeLine( 0, y, size.x, 1, buf );
}

/*----------------------------------------------------------------------------------------------*/
/* Constructor */
TAdvMsgBox::TAdvMsgBox(const char *title, const char *txt) : TWindow(TRect(0,0,10,20), title, 0), TWindowInit(&TAdvMsgBox::initFrame) {

   /* Calculate the exact size of the window wich is needed for
      the text to fit into it. */
   TRect rWin = this->calcRect(txt);

   /* Resize the window to the calculated size. */
   this->changeBounds(rWin);

   /* Create the vertical and horizontal scrollbar rectangles.
      The position/size of each one is based on the calculated
      size of the window. */
   TRect r_hs(rWin.a.x+1, rWin.b.y-1, rWin.b.x-1, rWin.b.y);
   TRect r_vs(rWin.b.x-1, rWin.a.y+1, rWin.b.x, rWin.b.y-1);

   /* Create the scrollbars. */
   TScrollBar *hScrollBar = new TScrollBar(r_hs);
   TScrollBar *vScrollBar = new TScrollBar(r_vs);

   rWin.grow(-1,-1);

   this->insert( new TTextBox(rWin, txt, m_nCols, m_nRows, hScrollBar, vScrollBar) );
   this->insert( hScrollBar );
   this->insert( vScrollBar );
}

/*----------------------------------------------------------------------------------------------*/
/* Calc Rect */
TRect& TAdvMsgBox::calcRect(const char *txt) {
   TRect& r = *new TRect(0,0,0,0);
   TRect b = TProgram::deskTop->getExtent();
   int i = 0;

   m_nCols = 0;
   m_nRows = 1;

   /* Calculate maximum number of columns and rows  */
   while( txt[i] != '\0' ) {
      if( txt[i] == '\n' ) {
         r.b.x = 0;
         m_nRows++;
         i++;
         continue;
      }

      if( txt[i] == '\t' )
         r.b.x += strlen(TAB_STRING);
      else
         r.b.x++;

      if(r.b.x > m_nCols)
         m_nCols = r.b.x;

      i++;
   }

   /* If too small, set e default minimum width.
      If too large, make it small enough to fit
      in the main window. */
   if( m_nCols < 12 )
      r.b.x = 16;
   else {
      r.b.x = m_nCols + 4;
      if( r.b.x > b.b.x )
         r.b.x = b.b.x;
   }

   /* If too small, set e default minimum height.
      If too large, make it small enough to fit
      in the main window. */
   if( m_nRows < 4 )
      r.b.y = 6;
   else
      r.b.y = m_nRows + 2;
      if( r.b.y > (b.b.y-2) )
         r.b.y = b.b.y-2;

   return r;
}

